%% Pop Gouy telescope variations
%
% What we'd like to do here is have a variety of Gouy phase telescopes,
% each spaced by, say,  10 degrees so we can match the ISC requirements.
% The telescopes start with the desired waist in between the sensors,  350
% um,  and propagate it 1 Rayleigh range,  plus an offset.   For each of a
% set of mirrors,  the rough overlap is calculated to match the input waist
% at a variety of positions. If the overlap is high enough,  we optimize
% that position.

%% Defines
lambda = 1.064e-6;
m  = 1;
cm = 1e-2;
mm = 1e-3;
um = 1e-6;

%% parameters
max_length = 1.8;
min_length = 10*cm;
stepsize = 10*cm;

% QPD target beam
w_target = 350 * um;
z_Rayleigh = pi*w_target^2/lambda;
x_offset = 10*cm;
q_target = beamq.beamWaistAndZ(w_target, 0);

% input beam
beamWidth = 3.5*mm;
radiusOfCurvature = -41.6*m;
q_pop = beamq.beamWaistAndR(beamWidth, radiusOfCurvature);

% Mirrors  -- a long but not complete set of ROC
mirror = [-2 -1 -0.5 0.5 1 2 inf];

% rough X array
x_1 = [z_Rayleigh+x_offset : stepsize : max_length];
x_i = [min_length : stepsize : max_length];

%% Calculate rough matches
%
% To calculate a rough match,  we set up a path for each mirror combination
% and for each combination of lengths.

% two mirror case
pop = beamPath;
pop.seedq = q_target;
pop.seedz = 0;
pop.targetq = q_pop;
pop.addComponent(component.curvedMirror(1,1,'M1'));
pop.addComponent(component.curvedMirror(1,1,'M2'));


data = [];
tic;
for ii = 1:length(mirror)
    m1 = mirror(ii);
    time_per = toc/ii;
    if ii > 1
        time_remaining = time_per * (length(mirror)-ii);
    else
        time_remaining = 1000;
    end
    
    fprintf(1, 'M1 is %d. %i seconds remaining.\n', m1, floor(time_remaining));
    for m2 = mirror
        for x1 = x_1
            for x2 = x_i
                pop.replaceComponent('M1', component.curvedMirror(m1, x1, 'M1'));
                pop.replaceComponent('M2', component.curvedMirror(m2, x1+x2, 'M2'));
                pop.targetz = x1+x2;
                overlap = pop.targetOverlap;
                data(end+1,:) = [m1 m2 x1 x2 overlap];
            end
        end
    end
end

%% real calculate
% 
% for each of the guys that works,  optimize the beam path

idx = find(data(:,5) > 0.2);
good_mirrors = unique(data(idx,[1 2]), 'rows');

clear newPop;
for ii = 1:length(good_mirrors)
    fprintf(1, 'Calculating %d out of %d\n', ii, length(good_mirrors));
    good_mirrors(ii,:)
    pop.replaceComponent('M1', component.curvedMirror(good_mirrors(ii,1), 1, 'M1'));
    pop.replaceComponent('M2', component.curvedMirror(good_mirrors(ii,2), 2, 'M2'));
    newPOP(end+1) = pop.optimizePath('M1', [z_Rayleigh+x_offset, max_length], ...
                               'M2', [z_Rayleigh+2*x_offset, 2*max_length], ...
                               'Target', [z_Rayleigh+3*x_offset 3*max_length]);
    goodPop(end+1) = newPOP(ii).targetOverlap;
end
